from .download import download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_file(xz_url + "BlenderAe_AeLayers.jsxbin",down_path)
download_file(xz_url + "BlenderAe_Help.jsxbin",down_path)
download_file(xz_url + "BlenderAe_Import.jsxbin",down_path)

import shutil
import subprocess
import sys
from pathlib import Path
import platform

import bpy
import blf

from .BlenderAe_op import (
    BLENDERAE_OT_connect_ae,
    BLENDERAE_OT_disconnect_ae,
    BLENDERAE_OT_export_ae,
    BLENDERAE_OT_help_ae,
    BLENDERAE_OT_import_ae_layers,
)
from .BlenderAe_Panel import VIEW3D_PT_blenderae
from .BlenderAe_props import BLAE_props

bl_info = {
    "name": "BlenderAe",
    "author": "Daniel Peterson <daniel@saltmedia.net>",
    "version": (1, 4, 5),
    "blender": (3, 0, 0),
    "location": "View3D",
    "description": "Export Blender data to After Effects",
    # "warning": "Requires installation of dependencies",
    "doc_url": "https://blenderae.com/",
    "category": "Import-Export",
}


def wrap_text(text: str, width: int) -> list[str]:
    '''
    Wrap the given text to the given width.

    Args:
        width: The maximum pixel width of each row.
        text: The text to be split and returned.

    Returns:
        A list of the split up text and empty space if necessary.
    '''
    return_text = []
    row_text = ''

    system = bpy.context.preferences.system
    dpi = 72 if system.ui_scale >= 1 else system.dpi
    blf.size(10, dpi)

    for word in text.split():
        word = f' {word}'
        line_len, _ = blf.dimensions(0, row_text + word)

        if line_len <= (width - 16):
            row_text += word
        else:
            return_text.append(row_text)
            row_text = word

    if row_text:
        return_text.append(row_text)

    return return_text

class BLENDERAEPrefs(bpy.types.AddonPreferences):
    # this must match the add-on name, use '__package__'
    # when defining this in a submodule of a python package.
    bl_idname = __package__

    dependencies_installed: bpy.props.StringProperty(
        name="Dependencies Installed",
        default="Click install, and restart Blender when complete.",
    )

    # print(bpy.context)
    # bpy.context.preferences.addons[__package__].preferences.dependencies_installed = "Click install, and restart Blender when complete."

    def draw(self, context):

        layout = self.layout
        # Check if the dependencies are installed and display a label
        try:
            # import psutil from the dependencies folder using pathlib
            addon_path = Path(__file__).parent.absolute()
            dependencies_path = addon_path / 'dependencies'
            sys.path.append(str(dependencies_path))
            import psutil

            bpy.context.preferences.addons[__package__].preferences.dependencies_installed = "Dependencies installed"
            dep_text = bpy.context.preferences.addons[__package__].preferences.dependencies_installed
            dep_icon = "FILE_TICK"

        except ImportError:
            dep_text = bpy.context.preferences.addons[__package__].preferences.dependencies_installed
            dep_icon = "ERROR"
            layout.label(text="These dependencies help BlenderAe to connect more reliably.", icon="INFO")

        col = self.layout.column()
        width = context.region.width
        ui_scale = context.preferences.system.ui_scale

        for text in wrap_text(dep_text, 0.8*(width/ui_scale)):
            col.label(text=text, icon=dep_icon)

        # layout.label(text=dep_text, icon=dep_icon)

        row1 = layout.row()
        row1.operator(
            BLENDERAE_OT_PIPinstall.bl_idname,
            text="Install Dependencies",
            icon="PACKAGE",
        )


        # row2 = layout.row()
        # row2.operator(
        #     BLENDERAE_OT_uninstall.bl_idname,
        #     text="Uninstall Dependencies",
        #     icon="PACKAGE",
        # )


class BLENDERAE_OT_PIPinstall(bpy.types.Operator):
    bl_idname = "blenderae.install_dependencies"
    bl_label = "Install Dependencies"
    bl_description = "Install the BlenderAe addon dependencies"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        # preferences = context.preferences
        # addon_prefs = preferences.addons[__name__].preferences
        def update_progress(current_task, total_tasks):
            wm = bpy.context.window_manager
            wm.progress_update(100 * current_task / total_tasks)

        wm = bpy.context.window_manager
        wm.progress_begin(0, 100)
        num_tasks = 5

        python_executable = sys.executable
        update_progress(1, num_tasks)
        subprocess.call([python_executable, "-m", "ensurepip", "--default-pip"])
        update_progress(2, num_tasks)
        subprocess.call([python_executable, "-m", "pip", "install", '--upgrade', "pip"])
        update_progress(3, num_tasks)

        # Get the addon path and install the requirements
        addon_path = Path(__file__).parent.absolute()
        dependencies_path = addon_path / "dependencies"
        requirements_path = addon_path / "requirements.txt"

        # Delete and recreate the dependencies folder
        shutil.rmtree(dependencies_path, ignore_errors=True)
        update_progress(4, num_tasks)
        dependencies_path.mkdir(exist_ok=True)
        update_progress(5, num_tasks)

        subprocess.call(
            [
                python_executable,
                "-m",
                "pip",
                "install",
                "-t",
                str(dependencies_path),
                "-r",
                str(requirements_path),
            ]
        )
        wm.progress_end()
        self.report({"INFO"}, "Installation complete, please restart Blender.")
        bpy.context.preferences.addons[__package__].preferences.dependencies_installed = "Dependencies installed. Restart Blender."
        return {"FINISHED"}

# Uninstall dependencies doesn't work because Windows doesn't allow deleting a folder that is in use.
# class BLENDERAE_OT_uninstall(bpy.types.Operator):
#     bl_idname = "blenderae.uninstall_dependencies"
#     bl_label = "Uninstall Dependencies"
#     bl_description = "Uninstall the BlenderAe addon dependencies"
#     bl_options = {"REGISTER", "UNDO"}

#     def execute(self, context):
#         # Get the addon path and install the requirements
#         addon_path = Path(__file__).parent.absolute()
#         dependencies_path = addon_path / "dependencies"

#         # Delete and recreate the dependencies folder
#         shutil.rmtree(dependencies_path, ignore_errors=False)
#         print(f"dependencies_path removed: {dependencies_path}")
#         print(dependencies_path.exists())
#         # dependencies_path.mkdir(exist_ok=True)

#         return {"FINISHED"}


classes = (
    VIEW3D_PT_blenderae,
    BLAE_props,
    BLENDERAE_OT_export_ae,
    BLENDERAE_OT_connect_ae,
    BLENDERAE_OT_disconnect_ae,
    BLENDERAE_OT_help_ae,
    BLENDERAE_OT_import_ae_layers,
    BLENDERAEPrefs,
    BLENDERAE_OT_PIPinstall,
    #BLENDERAE_OT_uninstall,
)


def register():
    for c in classes:
        bpy.utils.register_class(c)

    bpy.types.Scene.blenderae = bpy.props.PointerProperty(type=BLAE_props)
    return


def unregister():
    for c in reversed(classes):
        bpy.utils.unregister_class(c)

    # Delete custom properties
    del bpy.types.Scene.blenderae
    return
